#ifndef BASEPRECONDITIONER_H
#define BASEPRECONDITIONER_H

#include <stdexcept>
#include "CreatorInterface.h"
#include "ITL_Preconditioner.h"
#include "Collection.h"

namespace AMDiS {

  template< typename Vector >
  struct CreatorInterface< ITL_BasePreconditioner< Vector > > {
    typedef typename AMDiS::Collection< Vector >::PreconditionMatrix PreconditionMatrix;
    virtual ITL_BasePreconditioner< Vector >* create(const PreconditionMatrix& )=0 /*{ return NULL; }*/ ;
  };

  template< typename Vector >
  class NullCreator< ITL_BasePreconditioner< Vector > > : 
    public CreatorInterface< ITL_BasePreconditioner< Vector > >
  {
    typedef typename AMDiS::Collection< Vector >::PreconditionMatrix PreconditionMatrix;
    typedef ITL_BasePreconditioner< Vector > BaseClass;
    /// Creates no object.
    BaseClass* create() 
    {
      TEST_EXIT(false)("should never class this function");
      return NULL; 
    }

    BaseClass* create(const PreconditionMatrix& ) {
      TEST_EXIT(false)("this is the wrong null creator\n");
      return NULL;
    }
    ///
    virtual bool isNullCreator() 
    { 
      return true; 
    }
  };

  typedef CreatorInterface< ITL_BasePreconditioner< MTLTypes::MTLVector > > PreconditionCreator;
  typedef ITL_BasePreconditioner< MTLTypes::MTLVector > BasePreconditioner;

}

#endif
